 #include <Servo.h>
#include <IRremote.h>

#define RECEIVER_PIN 12
IRrecv receiver(RECEIVER_PIN); // create a receiver object of the IRrecv class
decode_results results; // create a results object of the decode_results class
unsigned long key_value = 0; // variable to store the pressed key value

// Motor A
int pwmA = 5;
int in1A = 3;
int in2A = 4;

// Motor Speed Values - Start at zero
int MotorSpeed1 = 0;

Servo servoWheel;
Servo servoSpool;

int servo_position = 0;

int button = 8;

void setup() {

  Serial.begin(9600);

  CLKPR = 0x00; //set power to normal useage  

  servoWheel.attach (9); 
  servoSpool.attach (10);
  servoWheel.write(servo_position);


  receiver.enableIRIn(); // enable the receiver
  receiver.blink13(true); // enable blinking of the built-in LED when an IR

  //for motor controller
  pinMode (pwmA, OUTPUT);
  pinMode (in1A, OUTPUT);
  pinMode (in2A, OUTPUT);

  pinMode(button, INPUT_PULLUP);

  liftWheel(); 
  
}

void doorLatchOpen(){
  //turn the door knob open
      servoSpool.attach (10);
      servoSpool.write(180);
      delay(4500);
      servoSpool.detach();
  }

void doorLatchClose(){
  //turn the door knob closed
      servoSpool.attach (10);
      servoSpool.write(0);
      delay(4100);
      servoSpool.detach();
  }

void forward(){ //to close door when open
  
   MotorSpeed1 = 255;
   digitalWrite(pwmA, MotorSpeed1);
   
   digitalWrite(in1A, LOW);
   digitalWrite(in2A, HIGH);
   delay(500); //drives door to door latch
   
   doorLatchOpen();
   delay(1300); //wait till door is in closed postion

   MotorSpeed1 = 0; 
   digitalWrite(pwmA, MotorSpeed1);
      
   doorLatchClose();
   
  }

void backwards(){ //to open door when closed
  
   doorLatchOpen();
   delay(1000); //wait till latch is open
   
   MotorSpeed1 = 200;
   digitalWrite(pwmA, MotorSpeed1);
   
   digitalWrite(in1A, HIGH);
   digitalWrite(in2A, LOW);
   
   delay(1300); // wait till door has cleared latch postion
   
   doorLatchClose();
   
   //delay(100); //drive till door is fully opened
   
   MotorSpeed1 = 0;
  digitalWrite(pwmA, MotorSpeed1);
  }

void liftWheel(){
    servoWheel.attach (9); 
    for (servo_position=33; servo_position >=0; servo_position -=1){
         delay(8);
         servoWheel.write(servo_position);
   }
}

void lowerWheel(){
  servoWheel.attach (9); 
   for (servo_position = 0; servo_position <= 44; servo_position +=1 ){   //pin 9 for yellow, middle is 5v, black or brown is ground 
      delay(8);
      servoWheel.write(servo_position);
  }
}

void backwardsPartOpen(){ //to open door when latched isnt closed
   
   MotorSpeed1 = 255;
   digitalWrite(pwmA, MotorSpeed1);
   
   digitalWrite(in1A, HIGH);
   digitalWrite(in2A, LOW);
   
   delay(3000); // wait till door has finshed moving

   MotorSpeed1 = 0;
  digitalWrite(pwmA, MotorSpeed1);
  }

  void forwardPartClosed(){ //to close door but not latch (partly Opened)
   
   MotorSpeed1 = 255;
   digitalWrite(pwmA, MotorSpeed1);
   
   digitalWrite(in1A, LOW);
   digitalWrite(in2A, HIGH);
   
   delay(3000); // wait till door has finshed moving

   MotorSpeed1 = 0;
  digitalWrite(pwmA, MotorSpeed1);
  }

void loop() { //main

 //turn off motor to stop power draw
   digitalWrite(in1A, LOW);
   digitalWrite(in2A, LOW);
   servoSpool.detach();
   servoWheel.detach();
   CLKPR = 0x04; 
   
  //Activate using button
   if(digitalRead(button) == LOW){
    
    CLKPR = 0x00; //set power to normal useage  
    
      //open door
      Serial.println("Openning (Manual)");
      lowerWheel();
      backwards();
      
      delay(5000);
        
      //closedoor
      Serial.println("Closing (Manual)");

      
      forward();
      liftWheel(); 
  }
  
 if (receiver.decode (&results)) { //if we have received an IR signal
//Serial.println (results.value, HEX); //display HEX results |

  CLKPR = 0x00; //set power to normal useage  

  if (results.value == 0xFFFFFFFF) {
      results.value = key_value; // set the value to the key value
      Serial.println(key_value);
    } 
    
    switch (results.value) { // compare the value to the following cases
      case 0xFF10EF:
        Serial.println("|<<"); //close door, when open
        Serial.println("Closing");
        lowerWheel();
        forward();
        liftWheel();
        break;

      case 0xFF5AA5:
        Serial.println(">>|"); // open door, when closed
        Serial.println("Openning");
        lowerWheel();
        backwards();
        liftWheel();
        break ;

        case 0xFF4AB5:
        Serial.println("UpButton|"); // open door, without latching
        lowerWheel();
        forwardPartClosed();
        liftWheel();
        break ;

        case 0xFF18E7:
        Serial.println("DownButton|"); // close door, without latch to open
        lowerWheel();
        backwardsPartOpen();
        liftWheel();
        break ;

        case 0xFF38C7: // "OK" button, open and close  
        Serial.println("Openning"); 
        lowerWheel();
        backwards();
        delay(5000);
        Serial.println("Closing");
        forward();
        liftWheel();  
        break;

        case 0xFF6897: //Star Button '*'
        Serial.println("Wheel Lifted"); 
        liftWheel();
        break;
       
    }
    key_value = results.value; // store the value as key_value
    receiver.resume(); // reset the receiver for the next code              
 }

  //turn off motor 
   digitalWrite(in1A, LOW);
   digitalWrite(in2A, LOW);
  CLKPR = 0x04;   //reduce power when not being used
}
